// noinspection DuplicatedCode

window._backup_console = window.console
window._backup_console_log = window.console.log

function _th_log(msg) {
    if (window.cc && window.cc.error) {
        window.cc.error(msg);
    } else if (window.cc && window.cc.warn) {
        window.cc.warn(msg);
    } else {
        window._backup_console_log.call(window._backup_console_log, msg)
    }
}

_th_log('all_in_once begin for url: ' + location.toString());

function in_frame() {
    try {
        return window.top !== window.self;
    } catch (e) {
        return true;
    }
}

function has_sub_frame() {
    return document.getElementsByTagName("iframe").length > 0;
}

function is_cocos() {
    return !!window.cc;
}

function is_egret() {
    return !!window.egret;
}

function get_infos() {
    return 'is_egret: ' + is_egret() + ', is_coos: ' + is_cocos() + ', has_sub_frame: ' +
        has_sub_frame() + ', in_frame: ' + in_frame() + ', in_game: ' + is_game();
}

function is_game() {
    return is_egret() || is_cocos() || !has_sub_frame();
}

function _tt_get_speed() {
    let speed = window.__speed || 1.0;
    const enable = window.__enable_speed !== false;
    if (!enable) {
        speed = 1.0;
    }
    return speed;
}

function _th_init_by_egret_timer() {
    const mem = {
        base: 0,
        rst: 0
    };

    function hack_time(origin) {
        if (!mem.base) {
            mem.base = mem.rst = origin
            return
        }
        const dx = origin - mem.base
        mem.base = origin
        const hacked_dx = dx * _tt_get_speed()
        mem.rst += hacked_dx
        return mem.rst
    }
    const egret_old_getTimer = egret.getTimer
    egret.getTimer = function () {
        const old = egret_old_getTimer()
        return hack_time(old)
    }
}

window._tt_has_init_inject = false
window._tt_has_tianxiace_inject = false
window._tt_has_init_calc_delta_time = false

function _init_inject() {
    if (window._tt_has_init_inject) {
        _th_log('_init_inject already injected!!')
        return
    }
    window._tt_has_init_inject = true
    _th_init_by_egret_timer();
}

function _send_msg_to_frame(msg) {
    for (const ele of document.getElementsByTagName('iframe')) {
        _th_log('send msg to child frame: ' + JSON.stringify(msg) + ", frame src: ", ele.src);
        if (ele.contentWindow) {
            ele.contentWindow.postMessage(msg, '*');
        }
    }
}

if (window.__speed !== null && window.__speed !== undefined) {
    window.__speed = 1.0;
}

window.setSpeed = function (speed) {
    _th_log('setSpeed called, ' + get_infos() + ", url: " + location.toString());
    _send_msg_to_frame({
        type: 'tt_setSpeed',
        speed
    });
    if (is_game()) {
        _th_log('in_game, so real set speed to: ' + speed + ', for url: ' + location.toString() + ", enabled: " + window.__enable_speed);
        _init_inject()
        window.__speed = speed;
    } else {
        _th_log('is not in game');
    }
};

window.setEnable = function (enable) {
    _send_msg_to_frame({
        type: 'tt_setEnable',
        enable
    });
    if (is_game()) {
        _init_inject()
        window.__enable_speed = enable;
    }
};

addEventListener('message', function (param) {
    if (param && param.data) {
        const data = param.data;
        if (data.type && data.type.startsWith('tt_')) {
            _th_log('receive msg: ' + JSON.stringify(data));
            if (data.type === 'tt_setSpeed') {
                setSpeed(data.speed);
            } else if (data.type === 'tt_setEnable') {
                setEnable(data.enable);
            }
        }
    }
}, false);

_th_log('all_in_once finish for url: ' + location.toString());
